@@ -34,7 +34,7 @@ except AttributeError:
             pass
 
 __author__ = 'Alec Thomas <alec@swapoff.org>'
-__version__ = '0.8.0'
+__version__ = '0.9.0'
 __version_tag__ = ''
 
 log = logging.getLogger('injector')
@@ -127,19 +127,22 @@ class Provider(object):
     __metaclass__ = ABCMeta
 
     @abstractmethod
-    def get(self):
+    def get(self, injector=None):
         raise NotImplementedError
 
 
 class ClassProvider(Provider):
     """Provides instances from a given class, created using an Injector."""
 
-    def __init__(self, cls, injector):
+    def __init__(self, cls, default_injector=None):
         self._cls = cls
-        self.injector = injector
+        self._injector = default_injector
 
-    def get(self):
-        return self.injector.create_object(self._cls)
+    def get(self, injector=None):
+        if injector is None:
+            warnings.warn("Injector object was not provided for the {!r}. Using legacy fallback method.".format(self))
+            injector = self._injector
+        return injector.create_object(self._cls)
 
 
 class CallableProvider(Provider):
@@ -167,8 +170,16 @@ class CallableProvider(Provider):
     def __init__(self, callable):
         self._callable = callable
 
-    def get(self):
-        return self._callable()
+    def get(self, injector=None):
+        if injector is None:
+            warnings.warn(
+                "Injector object was not provided for the {!r}. Using legacy fallback method.".format(
+                    self
+                )
+            )
+            return self._callable()
+        else:
+            return injector.call_with_injection(self._callable)
 
 
 class InstanceProvider(Provider):
@@ -191,7 +202,7 @@ class InstanceProvider(Provider):
     def __init__(self, instance):
         self._instance = instance
 
-    def get(self):
+    def get(self, injector=None):
         return self._instance
 
 
@@ -205,25 +216,25 @@ class ListOfProviders(Provider):
     def append(self, provider):
         self._providers.append(provider)
 
-    def get(self):
-        return [provider.get() for provider in self._providers]
+    def get(self, injector=None):
+        return [provider.get(injector) for provider in self._providers]
 
 
 class MultiBindProvider(ListOfProviders):
     """Used by :meth:`Binder.multibind` to flatten results of providers that
     return sequences."""
 
-    def get(self):
-        return [i for provider in self._providers for i in provider.get()]
+    def get(self, injector):
+        return [i for provider in self._providers for i in provider.get(injector)]
 
 
 class MapBindProvider(ListOfProviders):
     """A provider for map bindings."""
 
-    def get(self):
+    def get(self, injector=None):
         map = {}
         for provider in self._providers:
-            map.update(provider.get())
+            map.update(provider.get(injector))
         return map
 
 
@@ -383,7 +394,12 @@ class Binder(object):
         return Binding(interface, provider, scope)
 
     def provider_for(self, interface, to=None):
-        if isinstance(to, Provider):
+        if isinstance(interface, ProviderOf):
+            if to is not None:
+                raise Exception('ProviderOf cannot be bound to anything')
+            return InstanceProvider(
+                BoundProvider(self.injector, interface.interface))
+        elif isinstance(to, Provider):
             return to
         elif isinstance(interface, Provider):
             return interface
@@ -392,12 +408,12 @@ class Binder(object):
                              types.BuiltinMethodType)):
             return CallableProvider(to)
         elif issubclass(type(to), type):
-            return ClassProvider(to, self.injector)
+            return ClassProvider(to, default_injector=self.injector)
         elif isinstance(interface, BoundKey):
             @inject(**interface.kwargs)
             def proxy(**kwargs):
                 return interface.interface(**kwargs)
-            return CallableProvider(lambda: self.injector.call_with_injection(proxy))
+            return CallableProvider(proxy)
         elif isinstance(interface, AssistedBuilder):
             builder = AssistedBuilderImplementation(self.injector, *interface)
             return InstanceProvider(builder)
@@ -406,7 +422,7 @@ class Binder(object):
         elif issubclass(type(interface), type) or isinstance(interface, (tuple, list)):
             if issubclass(interface, (BaseKey, BaseMappingKey, BaseSequenceKey)) and to is not None:
                 return InstanceProvider(to)
-            return ClassProvider(interface, self.injector)
+            return ClassProvider(interface)
         elif hasattr(interface, '__call__'):
             function = to or interface
             if hasattr(function, '__bindings__'):
@@ -503,7 +519,7 @@ class SingletonScope(Scope):
 
     >>> class A(object): pass
     >>> injector = Injector()
-    >>> provider = ClassProvider(A, injector)
+    >>> provider = ClassProvider(A)
     >>> singleton = SingletonScope(injector)
     >>> a = singleton.get(A, provider)
     >>> b = singleton.get(A, provider)
@@ -518,7 +534,7 @@ class SingletonScope(Scope):
         try:
             return self._context[key]
         except KeyError:
-            provider = InstanceProvider(provider.get())
+            provider = InstanceProvider(provider.get(self.injector))
             self._context[key] = provider
             return provider
 
@@ -535,7 +551,7 @@ class ThreadLocalScope(Scope):
         try:
             return getattr(self._locals, repr(key))
         except AttributeError:
-            provider = InstanceProvider(provider.get())
+            provider = InstanceProvider(provider.get(self.injector))
             setattr(self._locals, repr(key), provider)
             return provider
 
@@ -646,14 +662,14 @@ class Injector(object):
         scope_key = BindingKey(scope)
         try:
             scope_binding = self.binder.get_binding(None, scope_key)
-            scope_instance = scope_binding.provider.get()
+            scope_instance = scope_binding.provider.get(self)
         except UnsatisfiedRequirement as e:
             raise Error('%s; scopes must be explicitly bound '
                         'with Binder.bind_scope(scope_cls)' % e)
 
         log.debug('%sInjector.get(%r, scope=%r) using %r',
                   self._log_prefix, interface, scope, binding.provider)
-        result = scope_instance.get(key, binding.provider).get()
+        result = scope_instance.get(key, binding.provider).get(self)
         log.debug('%s -> %r', self._log_prefix, result)
         return result
 
@@ -1120,3 +1136,43 @@ def _describe(c):
     if type(c) in (tuple, list):
         return '[%s]' % c[0].__name__
     return str(c)
+
+
+class ProviderOf(object):
+    """Can be used to get a :class:`BoundProvider` of an interface, for example:
+
+        >>> def provide_int():
+        ...     print('providing')
+        ...     return 123
+        >>>
+        >>> def configure(binder):
+        ...     binder.bind(int, to=provide_int)
+        >>>
+        >>> injector = Injector(configure)
+        >>> provider = injector.get(ProviderOf(int))
+        >>> type(provider)
+        <class 'injector.BoundProvider'>
+        >>> value = provider.get()
+        providing
+        >>> value
+        123
+    """
+
+    def __init__(self, interface):
+        self.interface = interface
+
+
+class BoundProvider(object):
+    """A :class:`Provider` tied with particular :class:`Injector` instance"""
+
+    def __init__(self, injector, interface):
+        self._injector = injector
+        self._interface = interface
+
+    def __repr__(self):
+        return 'BoundProvider(%r, %r)' % (
+            type(self).__name__, self._injector, self._interface)
+
+    def get(self):
+        """Get an implementation for the specified interface."""
+        return self._injector.get(self._interface)
